﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Net;
using System.IO;

namespace MinecraftInfoLib
{
    public sealed class Authentificator
    {

        private const string loginUrl = "https://login.minecraft.net/";
        private const int launcherVersion = 13;

        private const string errorBadLogin = "bad login";
        private const string errorUserNotPremium = "user not premium";

        private string username;
        private string password;

        public string Username
        {
            get { return username; }
            set { username = value; }
        }

        public string Password
        {
            get { return password; }
            set { password = value; }
        }

        public Authentificator()
        {
        }

        public Authentificator(string username_, string password_)
            : this()
        {
            username = username_;
            password = password_;
        }

        public AuthInfo Login()
        {
            WebRequest req = WebRequest.Create(loginUrl);

            string postDataString = String.Format(
                "user={0}&password={1}&version={2}",
                username, password, launcherVersion);
            byte[] postData = Encoding.ASCII.GetBytes(postDataString);

            req.Proxy = null;
            req.Method = "POST";
            req.ContentType = "application/x-www-form-urlencoded";
            req.ContentLength = postData.Length;

            Stream reqStream = req.GetRequestStream();
            reqStream.Write(postData, 0, postData.Length);
            reqStream.Close();

            WebResponse res = req.GetResponse();
            Stream resStream = res.GetResponseStream();
            StreamReader reader = new StreamReader(resStream);

            string[] ret = reader.ReadToEnd().Split(new char[] { ':' }, StringSplitOptions.RemoveEmptyEntries);

            resStream.Dispose();
            res.Close();

            AuthInfo info;

            if (ret.Length == 4)
            {
                info = new AuthInfo(
                    AuthInfo.ResponseTypes.Success,
                    ret[0],
                    ret[2],
                    ret[3]);
            }
            else if (ret[0].Equals(errorBadLogin, StringComparison.InvariantCultureIgnoreCase))
            {
                info = new AuthInfo(
                    AuthInfo.ResponseTypes.BadLogin,
                    null,
                    null,
                    null);
            }
            else if (ret[0].Equals(errorUserNotPremium, StringComparison.InvariantCultureIgnoreCase))
            {
                info = new AuthInfo(
                    AuthInfo.ResponseTypes.UserNotPremium,
                    null,
                    null,
                    null);
            }
            else
            {
                info = new AuthInfo(
                    AuthInfo.ResponseTypes.Unknown,
                    null,
                    null,
                    null);
            }

            return info;
        }

    }

    public sealed class AuthInfo
    {

        private readonly ResponseTypes responseType;

        private readonly string currentGameVersion;
        private readonly string username;
        private readonly string sessionId;

        public ResponseTypes ResponseType
        {
            get { return responseType; }
        }

        public string CurrentGameVersion
        {
            get { return currentGameVersion; }
        }

        public string Username
        {
            get { return username; }
        }

        public string SessionId
        {
            get { return sessionId; }
        }

        internal AuthInfo(ResponseTypes responseType_, string currentGameVersion_, string username_, string sessionId_)
        {
            responseType = responseType_;
            currentGameVersion = currentGameVersion_;
            username = username_;
            sessionId = sessionId_;
        }

        public override string ToString()
        {
            if (responseType == ResponseTypes.Success)
            {
                return String.Format(
                    "{0}:deprecated:{1}:{2}",
                    currentGameVersion,
                    username,
                    sessionId);
            }
            else
            {
                return responseType.ToString();
            }
        }

        public enum ResponseTypes
        {
            Success,
            BadLogin,
            UserNotPremium,
            Unknown
        }

    }
}
