﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;

namespace MinecraftInfoLib
{
    internal class Packet : IDisposable
    {

        #region Private Fields

        private MemoryStream stream;
        private bool isDisposed = false;

        private const string ex_NotSupportedYet = "Not supported yet.";

        #endregion

        #region Properties

        public long Position
        {
            get
            {
                return stream.Position;
            }
            set
            {
                stream.Position = value;
            }
        }

        public long Length
        {
            get
            {
                return stream.Length;
            }
        }

        #endregion

        #region Constructors

        public Packet(byte[] source)
        {
            stream = new MemoryStream(source);
        }

        public Packet()
        {
            stream = new MemoryStream();
        }

        #endregion

        #region Instance Methods

        #region Conversion

        public byte[] ToArray()
        {
            return stream.ToArray();
        }

        #endregion

        #region Writing

        public void Write(byte byte_)
        {
            stream.WriteByte(byte_);
        }

        public void Write(byte[] bytes_)
        {
            Write(bytes_, 0, bytes_.Length);
        }

        public void Write(byte[] bytes_, int offset_, int count_)
        {
            stream.Write(bytes_, offset_, count_);
        }

        public void Write(int integer_)
        {
            ToBeAndWrite(BitConverter.GetBytes(integer_));
        }

        public void Write(short short_)
        {
            ToBeAndWrite(BitConverter.GetBytes(short_));
        }

        public void Write(long long_)
        {
            ToBeAndWrite(BitConverter.GetBytes(long_));
        }

        public void Write(float float_)
        {
            throw new NotSupportedException(ex_NotSupportedYet);
        }

        public void Write(double double_)
        {
            throw new NotSupportedException(ex_NotSupportedYet);
        }

        public void Write(string string_)
        {
            Write((short)string_.Length);
            Write(Encoding.BigEndianUnicode.GetBytes(string_));
        }

        #endregion

        #region Reading

        public byte ReadByte()
        {
            return (byte)stream.ReadByte();
        }

        public byte[] ReadBytes(int count_)
        {
            byte[] buffer = new byte[count_];
            stream.Read(buffer, 0, count_);
            return buffer;
        }

        public short ReadShort()
        {
            return BitConverter.ToInt16(ReadAndConvert(2), 0);
        }

        public int ReadInt()
        {
            return BitConverter.ToInt32(ReadAndConvert(4), 0);
        }

        public long ReadLong()
        {
            return BitConverter.ToInt64(ReadAndConvert(8), 0);
        }

        public float ReadFloat()
        {
            throw new NotSupportedException(ex_NotSupportedYet);
        }

        public double ReadDouble()
        {
            throw new NotSupportedException(ex_NotSupportedYet);
        }

        public string ReadString()
        {
            short length = ReadShort();
            byte[] data = ReadBytes(length * 2);

            return Encoding.BigEndianUnicode.GetString(data);
        }

        #endregion

        #region Private Helper Methods

        private void ToBeAndWrite(byte[] byte_)
        {
            byte_ = EndianHelper.ReverseIfNeeded(byte_);
            stream.Write(byte_, 0, byte_.Length);
        }

        private byte[] ReadAndConvert(int count_)
        {
            byte[] buffer = new byte[count_];
            stream.Read(buffer, 0, count_);
            buffer = EndianHelper.ReverseIfNeeded(buffer);

            return buffer;
        }

        #endregion

        public void Dispose()
        {
            if (!isDisposed)
            {
                stream.Dispose();
            }
        }

        #endregion

    }
}
