﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;

namespace MinecraftInfoLib
{
    internal sealed class PacketFormatter
    {

        public enum DataType
        {
            Byte,
            Short,
            Integer,
            Long,
            Float,
            Double,
            String
        }

        public static object[] FromByteArray(byte[] data_, params DataType[] dataTypes_)
        {
            List<object> data = new List<object>();

            using (MemoryStream stream = new MemoryStream(data_))
            {
                for (int i = 0; i < dataTypes_.Length; i++)
                {
                    DataType current = dataTypes_[i];
                    int available = (int)(stream.Length - stream.Position);

                    if (available == 0) // if at end of stream
                    {
                        if (i == (dataTypes_.Length - 1)) // if last datatype reached
                        {
                            return data.ToArray(); // success
                        }
                        else
                        {
                            return null; // failure
                        }
                    }
                    else
                    {

                        if (current == DataType.Byte)
                        {
                            if (available >= 1)
                            {
                                data.Add(stream.ReadByte());
                            }
                            else
                            {
                                return null; // fail
                            }
                        }
                        else if (current == DataType.Short)
                        {
                            if (available >= 2)
                            {
                                byte[] buffer = new byte[2];
                                stream.Read(buffer, 0, 2);
                                buffer = EndianHelper.ReverseIfNeeded(buffer);

                                data.Add(BitConverter.ToInt16(buffer, 0));
                            }
                            else
                            {
                                return null;
                            }
                        }
                        else if (current == DataType.Integer)
                        {
                            if (available >= 4)
                            {
                                byte[] buffer = new byte[4];
                                stream.Read(buffer, 0, 4);
                                buffer = EndianHelper.ReverseIfNeeded(buffer);

                                data.Add(BitConverter.ToInt32(buffer, 0));
                            }
                            else
                            {
                                return null;
                            }
                        }
                        else if (current == DataType.Long)
                        {
                            if (available >= 8)
                            {
                                byte[] buffer = new byte[8];
                                stream.Read(buffer, 0, 8);
                                buffer = EndianHelper.ReverseIfNeeded(buffer);

                                data.Add(BitConverter.ToInt64(buffer, 0));
                            }
                            else
                            {
                                return null;
                            }
                        }
                        else if (current == DataType.Float || current == DataType.Double)
                        {
                            throw new NotSupportedException("Float and Double are not supported yet.");
                        }
                        else if (current == DataType.String)
                        {
                            if (available >= 2)
                            {
                                byte[] buffer = new byte[2];
                                stream.Read(buffer, 0, 2);
                                buffer = EndianHelper.ReverseIfNeeded(buffer);
                                Int16 length = BitConverter.ToInt16(buffer, 0);

                                if (available >= 2 + (length * 2))
                                {
                                    buffer = new byte[length * 2];
                                    stream.Read(buffer, 0, length * 2);
                                    string str = Encoding.BigEndianUnicode.GetString(buffer);

                                    data.Add(str);
                                }
                                else
                                {
                                    return null;
                                }
                            }
                            else
                            {
                                return null;
                            }
                        }
                    }
                }

                return null;
            }
        }

        public static byte[] ToByteArray(params object[] data_)
        {
            List<byte> bytes = new List<byte>();

            foreach (object obj in data_)
            {
                if (obj is byte)
                {
                    bytes.Add((byte)obj);
                }
                else if (obj is Int16)
                {
                    bytes.AddRange(EndianHelper.ReverseIfNeeded(BitConverter.GetBytes((Int16)obj)));
                }
                else if (obj is Int32)
                {
                    bytes.AddRange(EndianHelper.ReverseIfNeeded(BitConverter.GetBytes((Int32)obj)));
                }
                else if (obj is Int64)
                {
                    bytes.AddRange(EndianHelper.ReverseIfNeeded(BitConverter.GetBytes((Int64)obj)));
                }
                else if (obj is Single || obj is Double)
                {
                    throw new NotSupportedException("Float and Double are not supported yet.");
                }
                else if (obj is String)
                {
                    string str = (string)obj;
                    Int16 strLen = (Int16)str.Length;
                    byte[] byteStr = Encoding.BigEndianUnicode.GetBytes((string)obj);
                    byte[] byteLen = EndianHelper.ReverseIfNeeded(BitConverter.GetBytes(strLen));
                    bytes.AddRange(byteLen);
                    bytes.AddRange(byteStr);
                }
            }

            return bytes.ToArray();
        }
        
    }
}
