﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using System.Net;

namespace MinecraftInfoLib
{
    internal class SessionCreator
    {

        private const string sessionFormat = "http://session.minecraft.net/game/joinserver.jsp?user={0}&sessionId={1}&serverId={2}";

        private AuthInfo authInfo;
        private string serverHash;

        private object lockAccess = new object();

        public AuthInfo AuthInfo
        {
            get { return authInfo; }
            set { lock (lockAccess) authInfo = value; }
        }

        public string ServerHash
        {
            get { return serverHash; }
            set { lock (lockAccess) serverHash = value; }
        }

        public SessionCreator()
        {
        }

        public SessionCreator(AuthInfo authInfo_, string serverHash_)
            : this()
        {
            authInfo = authInfo_;
            serverHash = serverHash_;
        }

        /// <summary>
        /// Tries to log in using the assigned authentification information.
        /// </summary>
        /// <returns></returns>
        public bool Login()
        {
            lock (lockAccess)
            {
                string url = String.Format(sessionFormat, authInfo.Username, authInfo.SessionId, serverHash);
                Console.WriteLine(url);
                WebRequest req = WebRequest.Create(url);

                req.Proxy = null;
                req.Method = "GET";

                WebResponse res = req.GetResponse();
                Stream resStream = res.GetResponseStream();
                StreamReader reader = new StreamReader(resStream);

                string responseText = reader.ReadToEnd();

                reader.Dispose();
                resStream.Dispose();
                res.Close();

                return (responseText.Equals("ok", StringComparison.InvariantCultureIgnoreCase));
            }
        }

        public IAsyncResult BeginLogin(AsyncCallback callback_)
        {
            Func<bool> login = Login;
            return login.BeginInvoke(callback_, login);
        }

        public bool EndLogin(IAsyncResult result_)
        {
            Func<bool> login = (Func<bool>)result_.AsyncState;
            return login.EndInvoke(result_);
        }

    }
}
